/*
 * Copyright (c) 2019. JetBrains s.r.o.
 * Use of this source code is governed by the MIT license that can be found in the LICENSE file.
 */

package jetbrains.datalore.plotDemo.model.plotConfig

import jetbrains.datalore.plot.parsePlotSpec
import jetbrains.datalore.plotDemo.model.SharedPieces

open class Polygons {
    fun plotSpecList(): List<MutableMap<String, Any>> {
        return listOf(
            basic(),
            join(),

            // flip coord
            join().apply {
                this["coord"] = mapOf(
                    "name" to "flip",
                    "flip" to true
                )
            }
        )
    }

    companion object {
        fun basic(): MutableMap<String, Any> {
            val spec = "{" +
                    "   'kind': 'plot'," +
                    //        "   'data': " + ourData +
                    //        "           ," +
                    "   'mapping': {" +
                    "             'x': 'x'," +
                    "             'y': 'y'" +
                    "           }," +
                    "   'layers': [" +
                    "               {" +
                    "                  'geom': 'polygon'," +
                    "                  'mapping': {" +
                    "                               'fill': 'value'," +
                    "                               'group': 'id'" +
                    "                              }" +
                    "               }" +
                    "           ]" +
                    "}"

            val plotSpec = HashMap(parsePlotSpec(spec))
            plotSpec["data"] = SharedPieces.samplePolygons()
            return plotSpec
        }

        fun join(): MutableMap<String, Any> {


            val spec = """
{
  "ggsize": {
    "width": 800,
    "height": 400
  },
  "theme": {
    "axis_title": "blank"
  },
  "kind": "plot",
  "scales": [
    {
      "name": "Population",
      "aesthetic": "fill",
      "low": "light_blue",
      "high": "dark_green",
      "scale_mapper_kind": "color_gradient"
    }
  ],
  "layers": [
    {
      "geom": "map",
      "data": {
        "country": ["Argentina", "Chile", "Falkland Is.", "Uruguay", "Brazil", "Bolivia", "Peru", "Colombia", "Venezuela", "Guyana", "Suriname", "Ecuador", "Paraguay"],
        "pop_est": [44293293, 17789267, 2931, 3360148, 207353391, 11138234, 31036656, 47698524, 31304016, 737718, 591919, 16290913, 6943739]
      },
      "mapping": {
        "fill": "pop_est"
      },
      "tooltips": {
        "tooltip_formats": [],
        "tooltip_title": "@country",
        "tooltip_lines": [
          "Population|^fill"
        ]
      },
      "map_data_meta": {
        "geodataframe": {
          "geometry": "geometry"
        }
      },
      "map": {
        "continent": ["South America", "South America", "South America", "South America", "South America", "South America", "South America", "South America", "South America", "South America", "South America", "South America", "South America"],
        "name": ["Argentina", "Chile", "Falkland Is.", "Uruguay", "Brazil", "Bolivia", "Peru", "Colombia", "Venezuela", "Guyana", "Suriname", "Ecuador", "Paraguay"],
        "iso_a3": ["ARG", "CHL", "FLK", "URY", "BRA", "BOL", "PER", "COL", "VEN", "GUY", "SUR", "ECU", "PRY"],
        "gdp_md_est": [ 879400.0, 436100.0, 281.8, 73250.0, 3081000.0, 78350.0, 410400.0, 688000.0, 468600.0, 6093.0, 8547.0, 182400.0, 64670.0],
        "geometry": [
          "{\"type\": \"MultiPolygon\", \"coordinates\": [[[[-68.63401022758323, -52.63637045887449], [-68.25, -53.1], [-67.75, -53.85], [-66.45, -54.45], [-65.05, -54.699999999999996], [-65.5, -55.2], [-66.45, -55.25], [-66.95992000000001, -54.896810000000016], [-67.56244, -54.87001], [-68.63335000000001, -54.869499999999995], [-68.63401022758323, -52.63637045887449]]], [[[-57.62513342958296, -30.21629485445426], [-57.87493730328188, -31.016556084926208], [-58.14244035504076, -32.044503676076154], [-58.13264767112145, -33.040566908502015], [-58.349611172098875, -33.26318897881541], [-58.42707414410439, -33.909454441057576], [-58.49544206402655, -34.43148976007008], [-57.22582963726366, -35.28802662530788], [-57.36235877137878, -35.977390232081476], [-56.73748735210545, -36.41312590916655], [-56.78828528504836, -36.901571547189334], [-57.74915686708346, -38.18387053807989], [-59.23185706240189, -38.720220228837235], [-61.23744523786564, -38.9284245745412], [-62.33595699731013, -38.827707208004334], [-62.125763108962936, -39.42410491308485], [-62.330530971919494, -40.17258635840034], [-62.145994432205214, -40.67689666113672], [-62.745802781816984, -41.0287614886121], [-63.77049475773255, -41.16678923926369], [-64.73208980981973, -40.80267709733515], [-65.11803524439158, -41.06431487402891], [-64.97856055363582, -42.05800099056934], [-64.3034079657425, -42.35901620866951], [-63.75594784204239, -42.043686618824495], [-63.458059048095876, -42.563138116222405], [-64.37880388045633, -42.87355844499969], [-65.18180396183975, -43.495380954767796], [-65.32882341171013, -44.501366062193696], [-65.5652689276616, -45.036785577169795], [-66.50996578638934, -45.03962778094586], [-67.29379391139247, -45.55189625425519], [-67.58054643418008, -46.30177296324257], [-66.59706641301729, -47.033924655953825], [-65.64102657740149, -47.23613453551193], [-65.98508826360079, -48.133289076531135], [-67.16617896184769, -48.697337334996945], [-67.81608761256643, -49.86966887797038], [-68.72874508327321, -50.26421843851883], [-69.13853919134777, -50.732510267947795], [-68.81556148952356, -51.771104011594126], [-68.14999487982038, -52.34998340612768], [-68.57154537624133, -52.299443855346226], [-69.49836218939609, -52.14276091263727], [-71.91480383979638, -52.0090223058659], [-72.32940385607407, -51.42595631287243], [-72.30997351753234, -50.67700977966632], [-72.97574683296469, -50.741450290734285], [-73.32805091011453, -50.378785088909915], [-73.4154357571201, -49.31843637471297], [-72.64824744331494, -48.87861825947683], [-72.33116085477201, -48.2442383766618], [-72.44735531278027, -47.73853281025352], [-71.91725847033024, -46.88483814879177], [-71.55200944689128, -45.5607329241771], [-71.65931555854536, -44.973688653341426], [-71.22277889675976, -44.784242852559416], [-71.32980078803622, -44.407521661151655], [-71.79362260607193, -44.207172133156064], [-71.46405615913051, -43.787611179378345], [-71.91542395698389, -43.40856454851745], [-72.14889807807856, -42.254888197601375], [-71.7468037584155, -42.05138640723598], [-71.91573401557763, -40.83233936947069], [-71.68076127794649, -39.808164157878046], [-71.41351660834906, -38.91602223079114], [-70.81466427273469, -38.55299529394074], [-71.11862504747549, -37.57682748794724], [-71.12188066270987, -36.65812387466232], [-70.36476925320164, -36.00508879978992], [-70.38804948594913, -35.16968759535949], [-69.81730912950152, -34.1935714657983], [-69.81477698431922, -33.273886000299825], [-70.0743993801536, -33.09120981214805], [-70.53506893581951, -31.36501026787031], [-69.91900834825194, -30.33633920666828], [-70.01355038112992, -29.367922865518572], [-69.65613033718317, -28.459141127233686], [-69.00123491074825, -27.52121388113618], [-68.29554155137043, -26.89933969493578], [-68.59479977077268, -26.506908868111296], [-68.38600114609736, -26.185016371365215], [-68.41765296087614, -24.51855478281688], [-67.32844295924417, -24.02530323659095], [-66.9852339341777, -22.98634856536284], [-67.1066735500636, -22.735924574476417], [-66.27333940292485, -21.83231047942072], [-64.96489213729461, -22.075861504812327], [-64.37702104354226, -22.79809132252354], [-63.986838141522476, -21.99364430103595], [-62.84646847192156, -22.03498544686945], [-62.685057135657885, -22.249029229422387], [-60.846564704009914, -23.880712579038292], [-60.02896603050403, -24.032796319273274], [-58.80712846539498, -24.77145924245331], [-57.77721716981794, -25.16233977630904], [-57.63366004091113, -25.60365650808164], [-58.61817359071975, -27.123718763947096], [-57.60975969097614, -27.395898532828387], [-56.486701626192996, -27.548499037386293], [-55.69584550639816, -27.387837009390864], [-54.78879492859505, -26.621785577096134], [-54.625290696823576, -25.739255466415514], [-54.13004960795439, -25.547639255477254], [-53.628348965048744, -26.124865004177472], [-53.64873531758789, -26.92347258881609], [-54.490725267135524, -27.47475676850579], [-55.16228634298457, -27.881915378533463], [-56.29089962423908, -28.852760512000895], [-57.62513342958296, -30.21629485445426]]]]}",
          "{\"type\": \"MultiPolygon\", \"coordinates\": [[[[-68.63401022758323, -52.63637045887449], [-68.63335000000001, -54.869499999999995], [-67.56244, -54.87001], [-66.95992000000001, -54.896810000000016], [-67.29102999999992, -55.30123999999995], [-68.14862999999991, -55.61183], [-68.63999081081187, -55.58001799908692], [-69.2321, -55.49905999999993], [-69.95808999999997, -55.19843000000003], [-71.00567999999998, -55.053830000000005], [-72.26390000000004, -54.49513999999999], [-73.28519999999997, -53.95751999999993], [-74.66253, -52.837489999999946], [-73.8381, -53.04743000000002], [-72.43417999999997, -53.71539999999999], [-71.10773, -54.07432999999992], [-70.59177999999986, -53.61582999999996], [-70.26747999999998, -52.93123000000003], [-69.34564999999992, -52.518299999999954], [-68.63401022758323, -52.63637045887449]]], [[[-69.59042375352405, -17.580011895419332], [-69.10024695501949, -18.260125420812678], [-68.96681840684187, -18.981683444904107], [-68.44222510443092, -19.40506845467143], [-68.75716712103375, -20.372657972904463], [-68.21991309271128, -21.494346612231865], [-67.82817989772273, -22.872918796482175], [-67.1066735500636, -22.735924574476417], [-66.9852339341777, -22.98634856536284], [-67.32844295924417, -24.02530323659095], [-68.41765296087614, -24.51855478281688], [-68.38600114609736, -26.185016371365215], [-68.59479977077268, -26.506908868111296], [-68.29554155137043, -26.89933969493578], [-69.00123491074825, -27.52121388113618], [-69.65613033718317, -28.459141127233686], [-70.01355038112992, -29.367922865518572], [-69.91900834825194, -30.33633920666828], [-70.53506893581951, -31.36501026787031], [-70.0743993801536, -33.09120981214805], [-69.81477698431922, -33.273886000299825], [-69.81730912950152, -34.1935714657983], [-70.38804948594913, -35.16968759535949], [-70.36476925320164, -36.00508879978992], [-71.12188066270987, -36.65812387466232], [-71.11862504747549, -37.57682748794724], [-70.81466427273469, -38.55299529394074], [-71.41351660834906, -38.91602223079114], [-71.68076127794649, -39.808164157878046], [-71.91573401557763, -40.83233936947069], [-71.7468037584155, -42.05138640723598], [-72.14889807807856, -42.254888197601375], [-71.91542395698389, -43.40856454851745], [-71.46405615913051, -43.787611179378345], [-71.79362260607193, -44.207172133156064], [-71.32980078803622, -44.407521661151655], [-71.22277889675976, -44.784242852559416], [-71.65931555854536, -44.973688653341426], [-71.55200944689128, -45.5607329241771], [-71.91725847033024, -46.88483814879177], [-72.44735531278027, -47.73853281025352], [-72.33116085477201, -48.2442383766618], [-72.64824744331494, -48.87861825947683], [-73.4154357571201, -49.31843637471297], [-73.32805091011453, -50.378785088909915], [-72.97574683296469, -50.741450290734285], [-72.30997351753234, -50.67700977966632], [-72.32940385607407, -51.42595631287243], [-71.91480383979638, -52.0090223058659], [-69.49836218939609, -52.14276091263727], [-68.57154537624133, -52.299443855346226], [-69.46128434922667, -52.29195077266391], [-69.9427795071062, -52.53793059037322], [-70.8451016913546, -52.89920052852571], [-71.00633216010525, -53.83325204220132], [-71.429794684521, -53.85645476030037], [-72.55794287788488, -53.53141000118449], [-73.7027567206629, -52.835069268607235], [-73.7027567206629, -52.835070076051494], [-74.94676347522517, -52.262753588419], [-75.2600260077785, -51.62935475037325], [-74.97663245308988, -51.0433956846157], [-75.47975419788355, -50.37837167745158], [-75.60801510283198, -48.67377288187184], [-75.18276974150216, -47.7119194476232], [-74.1265809801047, -46.93925343199511], [-75.64439531116545, -46.64764332457207], [-74.69215369332312, -45.76397633238103], [-74.35170935738425, -44.10304412208794], [-73.24035600451522, -44.454960625995604], [-72.7178039211798, -42.38335580827898], [-73.38889990913822, -42.117532240569574], [-73.70133561877488, -43.365776462579774], [-74.33194312203261, -43.22495818458442], [-74.0179571194272, -41.79481292090683], [-73.67709937202999, -39.94221282324317], [-73.21759253609065, -39.25868865331856], [-73.50555945503712, -38.282882582351114], [-73.58806087919109, -37.15628468195598], [-73.1667170884993, -37.12378020604439], [-72.55313696968174, -35.50884002049106], [-71.86173214383263, -33.90909270603153], [-71.4384504869299, -32.41889942803078], [-71.66872066922247, -30.920644626592495], [-71.37008256700773, -30.09568206148503], [-71.48989437527645, -28.861442152625923], [-70.90512386746161, -27.640379734001247], [-70.72495398627599, -25.705924167587256], [-70.40396582709502, -23.628996677344574], [-70.09124589708074, -21.39331918710126], [-70.16441972520605, -19.756468194256165], [-70.37257239447771, -18.34797535570887], [-69.85844356960587, -18.092693780187012], [-69.59042375352405, -17.580011895419332]]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-61.2, -51.85], [-60.0, -51.25], [-59.15, -51.5], [-58.550000000000004, -51.10000000000001], [-57.75, -51.55], [-58.050000000000004, -51.900000000000006], [-59.400000000000006, -52.199999999999996], [-59.85000000000001, -51.85], [-60.7, -52.300000000000004], [-61.2, -51.85]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-57.62513342958296, -30.21629485445426], [-56.976025763564735, -30.109686374636127], [-55.97324459494094, -30.883075860316303], [-55.601510179249345, -30.853878676071393], [-54.57245154480512, -31.494511407193748], [-53.78795162618219, -32.047242526987624], [-53.209588995971544, -32.727666110974724], [-53.6505439927181, -33.20200408298183], [-53.373661668498244, -33.768377780900764], [-53.806425950726535, -34.39681487400223], [-54.93586605489773, -34.952646579733624], [-55.67408972840329, -34.75265878676407], [-56.21529700379607, -34.85983570733742], [-57.1396850246331, -34.430456231424245], [-57.81786068381551, -34.4625472958775], [-58.42707414410439, -33.909454441057576], [-58.349611172098875, -33.26318897881541], [-58.13264767112145, -33.040566908502015], [-58.14244035504076, -32.044503676076154], [-57.87493730328188, -31.016556084926208], [-57.62513342958296, -30.21629485445426]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-53.373661668498244, -33.768377780900764], [-53.6505439927181, -33.20200408298183], [-53.209588995971544, -32.727666110974724], [-53.78795162618219, -32.047242526987624], [-54.57245154480512, -31.494511407193748], [-55.601510179249345, -30.853878676071393], [-55.97324459494094, -30.883075860316303], [-56.976025763564735, -30.109686374636127], [-57.62513342958296, -30.21629485445426], [-56.29089962423908, -28.852760512000895], [-55.16228634298457, -27.881915378533463], [-54.490725267135524, -27.47475676850579], [-53.64873531758789, -26.92347258881609], [-53.628348965048744, -26.124865004177472], [-54.13004960795439, -25.547639255477254], [-54.625290696823576, -25.739255466415514], [-54.42894609233059, -25.162184747012166], [-54.29347632507745, -24.570799655863965], [-54.29295956075452, -24.02101409271073], [-54.65283423523513, -23.83957813893396], [-55.02790178080955, -24.00127369557523], [-55.40074723979542, -23.956935316668805], [-55.517639329639636, -23.571997572526637], [-55.610682745981144, -22.655619398694846], [-55.79795813660691, -22.356929620047822], [-56.47331743022939, -22.086300144135283], [-56.8815095689029, -22.28215382252148], [-57.937155727761294, -22.090175876557172], [-57.8706739976178, -20.73268767668195], [-58.166392381408045, -20.176700941653678], [-57.85380164247451, -19.96999521248619], [-57.949997321185826, -19.40000416430682], [-57.67600887717431, -18.96183969490403], [-57.49837114117099, -18.174187513911292], [-57.734558274961, -17.55246835700777], [-58.28080400250225, -17.271710300366017], [-58.38805843772404, -16.877109063385276], [-58.24121985536668, -16.299573256091293], [-60.158389655179036, -16.258283786690086], [-60.54296566429515, -15.093910414289596], [-60.251148851142936, -15.07721892665932], [-60.26432634137737, -14.645979099183641], [-60.45919816755003, -14.354007256734555], [-60.503304002511136, -13.775954685117659], [-61.08412126325565, -13.479383640194598], [-61.71320431176078, -13.489202162330052], [-62.127080857986385, -13.198780612849724], [-62.803060268796386, -13.000653171442686], [-63.19649878605057, -12.627032565972435], [-64.3163529120316, -12.461978041232193], [-65.40228146021303, -11.566270440317155], [-65.32189876978302, -10.895872084194679], [-65.44483700220539, -10.511451104375432], [-65.33843522811642, -9.761987806846392], [-66.6469083319628, -9.931331475466862], [-67.17380123561074, -10.306812432499612], [-68.04819230820539, -10.712059014532485], [-68.27125362819326, -11.01452117273682], [-68.78615759954948, -11.03638030359628], [-69.52967810736496, -10.951734307502194], [-70.0937522040469, -11.123971856331012], [-70.54868567572841, -11.009146823778465], [-70.48189388699117, -9.490118096558845], [-71.30241227892154, -10.079436130415374], [-72.18489071316985, -10.053597914269432], [-72.56303300646564, -9.520193780152717], [-73.22671342639016, -9.462212823121234], [-73.01538265653255, -9.032833347208062], [-73.57105933296707, -8.424446709835834], [-73.98723548042966, -7.523829847853065], [-73.7234014553635, -7.340998630404414], [-73.72448666044164, -6.91859547285064], [-73.1200274319236, -6.629930922068239], [-73.21971126981461, -6.089188734566078], [-72.9645072089412, -5.7412513159448935], [-72.89192765978726, -5.274561455916981], [-71.74840572781655, -4.593982842633011], [-70.92884334988358, -4.401591485210368], [-70.7947688463023, -4.251264743673303], [-69.89363521999663, -4.2981869441943275], [-69.44410193548961, -1.5562871232198177], [-69.42048580593223, -1.1226185034264091], [-69.5770653957766, -0.549991957200163], [-70.02065589057005, -0.18515634521953928], [-70.01556576198931, 0.5414142928042054], [-69.45239600287246, 0.7061587589506929], [-69.25243404811906, 0.6026508650700748], [-69.21863766140018, 0.9856765812174331], [-69.80459672715773, 1.0890811222334662], [-69.81697323269162, 1.7148052026396243], [-67.86856502955884, 1.6924551456733923], [-67.5378100246747, 2.03716278727633], [-67.2599975246736, 1.7199986840849562], [-67.0650481838525, 1.130112209473225], [-66.87632585312258, 1.253360500489336], [-66.32576514348496, 0.7244522159820121], [-65.54826738143757, 0.7892544620760303], [-65.35471330428837, 1.0952822941085003], [-64.61101192895987, 1.3287305769870417], [-64.19930579289051, 1.49285492594602], [-64.08308549666609, 1.9163691267940803], [-63.368788011311665, 2.200899562993129], [-63.42286739770512, 2.4110676131241746], [-64.2699991522658, 2.497005520025567], [-64.40882788761792, 3.126786200366624], [-64.3684944322141, 3.797210394705246], [-64.81606401229402, 4.056445217297423], [-64.62865943058755, 4.14848094320925], [-63.88834286157416, 4.020530096854571], [-63.093197597899106, 3.7705711938587854], [-62.804533047116706, 4.006965033377952], [-62.08542965355913, 4.162123521334308], [-60.96689327660154, 4.536467596856639], [-60.601179165271944, 4.91809804933213], [-60.73357418480372, 5.200277207861901], [-60.21368343773133, 5.244486395687602], [-59.980958624904886, 5.014061184098139], [-60.11100236676738, 4.574966538914083], [-59.767405768458715, 4.423502915866607], [-59.53803992373123, 3.9588025984819377], [-59.815413174057866, 3.6064985213320853], [-59.97452490908456, 2.755232652188056], [-59.71854570172675, 2.2496304386443597], [-59.64604366722126, 1.786893825686789], [-59.03086157900265, 1.3176976586927225], [-58.540012986878295, 1.2680882836925207], [-58.429477098205965, 1.4639419620787208], [-58.11344987652502, 1.5071951359070253], [-57.66097103537737, 1.6825849471056387], [-57.335822923396904, 1.9485377058957594], [-56.78270423036083, 1.8637108422886541], [-56.539385748914555, 1.8995226098669207], [-55.995698004771754, 1.8176671411166012], [-55.905600145070885, 2.0219957543986595], [-56.0733418442903, 2.2207949894254995], [-55.973322109589375, 2.510363877773017], [-55.569755011606, 2.4215062524471307], [-55.09758744975514, 2.5237480737366127], [-54.524754197799716, 2.3118488631237852], [-54.08806250671725, 2.105556545414629], [-53.77852067728892, 2.3767027856500818], [-53.554839240113544, 2.334896551925951], [-53.41846513529531, 2.0533891870159806], [-52.939657151894956, 2.1248576928756364], [-52.55642473001842, 2.504705308437053], [-52.249337531123956, 3.241094468596245], [-51.65779741067889, 4.156232408053029], [-51.31714636901086, 4.203490505383954], [-51.069771287629656, 3.650397650564031], [-50.508875291533656, 1.901563828942457], [-49.97407589374506, 1.736483465986069], [-49.94710079608871, 1.0461896834312228], [-50.699251268096916, 0.22298411702168153], [-50.38821082213214, -0.07844451253681939], [-48.62056677915632, -0.2354891902718208], [-48.58449662941659, -1.2378052710050014], [-47.824956427590635, -0.5816179337628], [-46.566583624851226, -0.941027520352776], [-44.905703090990414, -1.551739597178134], [-44.417619187993665, -2.137750339367976], [-44.58158850765578, -2.691308282078524], [-43.418791266440195, -2.383110039889793], [-41.47265682632825, -2.9120183243971165], [-39.97866533055404, -2.873054294449041], [-38.50038347019657, -3.7006523576033956], [-37.2232521225352, -4.820945733258917], [-36.45293738457639, -5.109403578312154], [-35.59779578301047, -5.149504489770649], [-35.23538896334756, -5.464937432480247], [-34.89602983248683, -6.738193047719711], [-34.729993455533034, -7.343220716992967], [-35.12821204277422, -8.996401462442286], [-35.636966518687714, -9.649281508017815], [-37.046518724097, -11.040721123908803], [-37.68361161960736, -12.171194756725823], [-38.42387651218844, -13.038118584854288], [-38.67388709161652, -13.057652276260619], [-38.953275722802545, -13.793369642800023], [-38.88229814304965, -15.667053724838768], [-39.16109249526431, -17.208406670808472], [-39.2673392400564, -17.867746270420483], [-39.58352149103423, -18.262295830968938], [-39.76082333022764, -19.59911345792741], [-40.77474077001034, -20.904511814052423], [-40.94475623225061, -21.93731698983781], [-41.754164191238225, -22.370675551037458], [-41.98828426773656, -22.970070489190896], [-43.07470374202475, -22.96769337330547], [-44.64781185563781, -23.351959323827842], [-45.35213578955992, -23.796841729428582], [-46.47209326840554, -24.088968601174543], [-47.64897233742066, -24.885199069927722], [-48.4954581365777, -25.877024834905654], [-48.64100480812774, -26.623697605090932], [-48.474735887228654, -27.17591196056189], [-48.661520351747626, -28.18613453543572], [-48.8884574041574, -28.674115085567884], [-49.587329474472675, -29.224469089476337], [-50.696874152211485, -30.98446502047296], [-51.576226162306156, -31.77769825615321], [-52.256081305538046, -32.24536996839467], [-52.712099982297694, -33.19657805759118], [-53.373661668498244, -33.768377780900764]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-69.52967810736496, -10.951734307502194], [-68.78615759954948, -11.03638030359628], [-68.27125362819326, -11.01452117273682], [-68.04819230820539, -10.712059014532485], [-67.17380123561074, -10.306812432499612], [-66.6469083319628, -9.931331475466862], [-65.33843522811642, -9.761987806846392], [-65.44483700220539, -10.511451104375432], [-65.32189876978302, -10.895872084194679], [-65.40228146021303, -11.566270440317155], [-64.3163529120316, -12.461978041232193], [-63.19649878605057, -12.627032565972435], [-62.803060268796386, -13.000653171442686], [-62.127080857986385, -13.198780612849724], [-61.71320431176078, -13.489202162330052], [-61.08412126325565, -13.479383640194598], [-60.503304002511136, -13.775954685117659], [-60.45919816755003, -14.354007256734555], [-60.26432634137737, -14.645979099183641], [-60.251148851142936, -15.07721892665932], [-60.54296566429515, -15.093910414289596], [-60.158389655179036, -16.258283786690086], [-58.24121985536668, -16.299573256091293], [-58.38805843772404, -16.877109063385276], [-58.28080400250225, -17.271710300366017], [-57.734558274961, -17.55246835700777], [-57.49837114117099, -18.174187513911292], [-57.67600887717431, -18.96183969490403], [-57.949997321185826, -19.40000416430682], [-57.85380164247451, -19.96999521248619], [-58.166392381408045, -20.176700941653678], [-58.183471442280506, -19.868399346600363], [-59.11504248720611, -19.3569060197754], [-60.04356462262649, -19.342746677327426], [-61.78632646345377, -19.633736667562964], [-62.2659612697708, -20.513734633061276], [-62.291179368729225, -21.051634616787393], [-62.685057135657885, -22.249029229422387], [-62.84646847192156, -22.03498544686945], [-63.986838141522476, -21.99364430103595], [-64.37702104354226, -22.79809132252354], [-64.96489213729461, -22.075861504812327], [-66.27333940292485, -21.83231047942072], [-67.1066735500636, -22.735924574476417], [-67.82817989772273, -22.872918796482175], [-68.21991309271128, -21.494346612231865], [-68.75716712103375, -20.372657972904463], [-68.44222510443092, -19.40506845467143], [-68.96681840684187, -18.981683444904107], [-69.10024695501949, -18.260125420812678], [-69.59042375352405, -17.580011895419332], [-68.9596353827533, -16.50069793057127], [-69.38976416693471, -15.660129082911652], [-69.16034664577495, -15.323973890853019], [-69.33953467474701, -14.953195489158832], [-68.9488866848366, -14.453639418193283], [-68.92922380234954, -13.602683607643009], [-68.88007951523997, -12.899729099176653], [-68.66507971868963, -12.561300144097173], [-69.52967810736496, -10.951734307502194]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-69.89363521999663, -4.2981869441943275], [-70.7947688463023, -4.251264743673303], [-70.92884334988358, -4.401591485210368], [-71.74840572781655, -4.593982842633011], [-72.89192765978726, -5.274561455916981], [-72.9645072089412, -5.7412513159448935], [-73.21971126981461, -6.089188734566078], [-73.1200274319236, -6.629930922068239], [-73.72448666044164, -6.91859547285064], [-73.7234014553635, -7.340998630404414], [-73.98723548042966, -7.523829847853065], [-73.57105933296707, -8.424446709835834], [-73.01538265653255, -9.032833347208062], [-73.22671342639016, -9.462212823121234], [-72.56303300646564, -9.520193780152717], [-72.18489071316985, -10.053597914269432], [-71.30241227892154, -10.079436130415374], [-70.48189388699117, -9.490118096558845], [-70.54868567572841, -11.009146823778465], [-70.0937522040469, -11.123971856331012], [-69.52967810736496, -10.951734307502194], [-68.66507971868963, -12.561300144097173], [-68.88007951523997, -12.899729099176653], [-68.92922380234954, -13.602683607643009], [-68.9488866848366, -14.453639418193283], [-69.33953467474701, -14.953195489158832], [-69.16034664577495, -15.323973890853019], [-69.38976416693471, -15.660129082911652], [-68.9596353827533, -16.50069793057127], [-69.59042375352405, -17.580011895419332], [-69.85844356960587, -18.092693780187012], [-70.37257239447771, -18.34797535570887], [-71.37525021023693, -17.773798516513857], [-71.46204077827113, -17.363487644116383], [-73.44452958850042, -16.359362888252996], [-75.23788265654144, -15.265682875227782], [-76.00920508492995, -14.649286390850321], [-76.42346920439775, -13.823186944232432], [-76.25924150257417, -13.535039157772943], [-77.10619238962184, -12.22271615972082], [-78.09215287953464, -10.377712497604065], [-79.03695309112695, -8.386567884965892], [-79.44592037628485, -7.93083342858386], [-79.76057817251005, -7.194340915560084], [-80.53748165558608, -6.541667575713717], [-81.24999630402642, -6.136834405139183], [-80.92634680858244, -5.690556735866565], [-81.41094255239946, -4.7367648250554595], [-81.09966956248937, -4.036394138203697], [-80.30256059438722, -3.4048564591647126], [-80.18401485870967, -3.8211617977080437], [-80.46929460317695, -4.0592867977089995], [-80.44224199087216, -4.425724379090674], [-80.02890804718561, -4.3460909969288934], [-79.62497921417618, -4.454198093283495], [-79.20528906931773, -4.959128513207389], [-78.63989722361234, -4.547784112164074], [-78.45068396677564, -3.873096612161376], [-77.83790483265861, -3.003020521663103], [-76.63539425322672, -2.6086776668438176], [-75.54499569365204, -1.5616097957458803], [-75.23372270374195, -0.9114169246495294], [-75.37322323271385, -0.1520317521204504], [-75.10662451852008, -0.05720549886486026], [-74.44160051135597, -0.5308200008198867], [-74.12239518908906, -1.002832533373848], [-73.6595035468346, -1.2604912247811342], [-73.07039221870724, -2.3089543595509525], [-72.32578650581365, -2.434218031426454], [-71.7747607082854, -2.169789727388938], [-71.41364579942979, -2.3428024227021282], [-70.81347571479196, -2.2568645158007428], [-70.04770850287485, -2.725156345229699], [-70.69268205430971, -3.742872002785859], [-70.39404395209499, -3.7665914852078255], [-69.89363521999663, -4.2981869441943275]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-66.87632585312258, 1.253360500489336], [-67.0650481838525, 1.130112209473225], [-67.2599975246736, 1.7199986840849562], [-67.5378100246747, 2.03716278727633], [-67.86856502955884, 1.6924551456733923], [-69.81697323269162, 1.7148052026396243], [-69.80459672715773, 1.0890811222334662], [-69.21863766140018, 0.9856765812174331], [-69.25243404811906, 0.6026508650700748], [-69.45239600287246, 0.7061587589506929], [-70.01556576198931, 0.5414142928042054], [-70.02065589057005, -0.18515634521953928], [-69.5770653957766, -0.549991957200163], [-69.42048580593223, -1.1226185034264091], [-69.44410193548961, -1.5562871232198177], [-69.89363521999663, -4.2981869441943275], [-70.39404395209499, -3.7665914852078255], [-70.69268205430971, -3.742872002785859], [-70.04770850287485, -2.725156345229699], [-70.81347571479196, -2.2568645158007428], [-71.41364579942979, -2.3428024227021282], [-71.7747607082854, -2.169789727388938], [-72.32578650581365, -2.434218031426454], [-73.07039221870724, -2.3089543595509525], [-73.6595035468346, -1.2604912247811342], [-74.12239518908906, -1.002832533373848], [-74.44160051135597, -0.5308200008198867], [-75.10662451852008, -0.05720549886486026], [-75.37322323271385, -0.1520317521204504], [-75.8014658271166, 0.08480133707320192], [-76.29231441924097, 0.4160472680641192], [-76.5763797675494, 0.256935533037435], [-77.4249843004304, 0.395686753741117], [-77.66861284047044, 0.8258930525709616], [-77.85506140817952, 0.8099250349927729], [-78.85525875518871, 1.380923773601822], [-78.99093522817104, 1.6913699405952514], [-78.61783138702371, 1.766404120283056], [-78.66211808949785, 2.2673554549204766], [-78.42761043975733, 2.629555568854215], [-77.93154252797149, 2.6966057397529255], [-77.51043128122501, 3.325016994638247], [-77.12768978545526, 3.8496361352653565], [-77.49627193877703, 4.087606105969428], [-77.3076012844794, 4.6679841170394525], [-77.53322058786573, 5.582811997902497], [-77.31881507028675, 5.84535411216136], [-77.47666073272228, 6.691116441266303], [-77.88157141794525, 7.223771267114785], [-77.7534138658614, 7.709839789252143], [-77.43110795765699, 7.638061224798734], [-77.24256649444008, 7.935278225125444], [-77.47472286651133, 8.524286200388218], [-77.35336076527386, 8.67050466555807], [-76.83667395700357, 8.638749497914716], [-76.08638383655786, 9.336820583529487], [-75.67460018584006, 9.443248195834599], [-75.66470414905618, 9.774003200718738], [-75.48042599150335, 10.618990383339309], [-74.90689510771199, 11.083044745320322], [-74.27675269234489, 11.102035834187587], [-74.1972226630477, 11.310472723836867], [-73.41476396350029, 11.22701528568548], [-72.62783525255963, 11.731971543825523], [-72.23819495307892, 11.955549628136326], [-71.75409013536864, 12.437303168177309], [-71.3998223537917, 12.376040757695293], [-71.13746110704588, 12.112981879113505], [-71.3315836249503, 11.776284084515808], [-71.97392167833829, 11.60867157637712], [-72.22757544624294, 11.10870209395324], [-72.61465776232521, 10.821975409381778], [-72.9052860175347, 10.450344346554772], [-73.02760413276957, 9.736770331252444], [-73.30495154488005, 9.151999823437606], [-72.7887298245004, 9.085027167187334], [-72.6604947577681, 8.625287787302682], [-72.43986223009796, 8.405275376820029], [-72.36090064155597, 8.002638454617895], [-72.47967892117885, 7.632506008327354], [-72.44448727078807, 7.423784898300482], [-72.19835242378188, 7.340430813013683], [-71.96017574734864, 6.991614895043539], [-70.67423356798152, 7.087784735538719], [-70.09331295437242, 6.96037649172311], [-69.38947994655712, 6.0998605411988365], [-68.98531856960236, 6.206804917826858], [-68.26505245631823, 6.153268133972475], [-67.69508724635502, 6.267318020040647], [-67.34143958196557, 6.095468044454023], [-67.52153194850275, 5.556870428891969], [-67.74469662135522, 5.221128648291668], [-67.82301225449355, 4.503937282728899], [-67.62183590358129, 3.8394817163199946], [-67.33756384954368, 3.5423422306417223], [-67.30317318385345, 3.31845408773718], [-67.8099381171237, 2.820655015469569], [-67.44709204778631, 2.6002808699608693], [-67.18129431829307, 2.250638129074062], [-66.87632585312258, 1.253360500489336]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-60.73357418480372, 5.200277207861901], [-60.601179165271944, 4.91809804933213], [-60.96689327660154, 4.536467596856639], [-62.08542965355913, 4.162123521334308], [-62.804533047116706, 4.006965033377952], [-63.093197597899106, 3.7705711938587854], [-63.88834286157416, 4.020530096854571], [-64.62865943058755, 4.14848094320925], [-64.81606401229402, 4.056445217297423], [-64.3684944322141, 3.797210394705246], [-64.40882788761792, 3.126786200366624], [-64.2699991522658, 2.497005520025567], [-63.42286739770512, 2.4110676131241746], [-63.368788011311665, 2.200899562993129], [-64.08308549666609, 1.9163691267940803], [-64.19930579289051, 1.49285492594602], [-64.61101192895987, 1.3287305769870417], [-65.35471330428837, 1.0952822941085003], [-65.54826738143757, 0.7892544620760303], [-66.32576514348496, 0.7244522159820121], [-66.87632585312258, 1.253360500489336], [-67.18129431829307, 2.250638129074062], [-67.44709204778631, 2.6002808699608693], [-67.8099381171237, 2.820655015469569], [-67.30317318385345, 3.31845408773718], [-67.33756384954368, 3.5423422306417223], [-67.62183590358129, 3.8394817163199946], [-67.82301225449355, 4.503937282728899], [-67.74469662135522, 5.221128648291668], [-67.52153194850275, 5.556870428891969], [-67.34143958196557, 6.095468044454023], [-67.69508724635502, 6.267318020040647], [-68.26505245631823, 6.153268133972475], [-68.98531856960236, 6.206804917826858], [-69.38947994655712, 6.0998605411988365], [-70.09331295437242, 6.96037649172311], [-70.67423356798152, 7.087784735538719], [-71.96017574734864, 6.991614895043539], [-72.19835242378188, 7.340430813013683], [-72.44448727078807, 7.423784898300482], [-72.47967892117885, 7.632506008327354], [-72.36090064155597, 8.002638454617895], [-72.43986223009796, 8.405275376820029], [-72.6604947577681, 8.625287787302682], [-72.7887298245004, 9.085027167187334], [-73.30495154488005, 9.151999823437606], [-73.02760413276957, 9.736770331252444], [-72.9052860175347, 10.450344346554772], [-72.61465776232521, 10.821975409381778], [-72.22757544624294, 11.10870209395324], [-71.97392167833829, 11.60867157637712], [-71.3315836249503, 11.776284084515808], [-71.36000566271082, 11.539993597861212], [-71.94704993354651, 11.423282375530022], [-71.62086829292019, 10.969459947142795], [-71.63306393094109, 10.446494452349029], [-72.07417395698451, 9.865651353388373], [-71.69564409044654, 9.072263088411248], [-71.26455929226773, 9.137194525585983], [-71.03999935574339, 9.859992784052409], [-71.35008378771079, 10.211935126176215], [-71.40062333849224, 10.968969021036015], [-70.15529883490652, 11.37548167566004], [-70.29384334988103, 11.846822414594214], [-69.94324459499683, 12.162307033736099], [-69.58430009629747, 11.459610907431212], [-68.88299923366445, 11.443384507691563], [-68.23327145045873, 10.885744126829946], [-68.19412655299763, 10.554653225135922], [-67.29624854192633, 10.54586823164631], [-66.227864142508, 10.648626817258688], [-65.65523759628175, 10.200798855017323], [-64.89045223657817, 10.0772146671913], [-64.32947872583374, 10.38959870039568], [-64.31800655786495, 10.64141795495398], [-63.07932247582873, 10.7017243514386], [-61.880946010980196, 10.715625311725104], [-62.73011898461641, 10.420268662960906], [-62.388511928950976, 9.94820445397464], [-61.58876746280194, 9.873066921422264], [-60.83059668643172, 9.381339829948942], [-60.67125240745973, 8.580174261911878], [-60.15009558779618, 8.602756862823426], [-59.758284878159195, 8.367034816924047], [-60.5505879380582, 7.779602972846178], [-60.637972785063766, 7.4149999048108555], [-60.2956680975624, 7.043911444522919], [-60.54399919294099, 6.856584377464883], [-61.15933631045648, 6.696077378766319], [-61.13941504580795, 6.234296779806144], [-61.410302903881956, 5.959068101419618], [-60.73357418480372, 5.200277207861901]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-56.539385748914555, 1.8995226098669207], [-56.78270423036083, 1.8637108422886541], [-57.335822923396904, 1.9485377058957594], [-57.66097103537737, 1.6825849471056387], [-58.11344987652502, 1.5071951359070253], [-58.429477098205965, 1.4639419620787208], [-58.540012986878295, 1.2680882836925207], [-59.03086157900265, 1.3176976586927225], [-59.64604366722126, 1.786893825686789], [-59.71854570172675, 2.2496304386443597], [-59.97452490908456, 2.755232652188056], [-59.815413174057866, 3.6064985213320853], [-59.53803992373123, 3.9588025984819377], [-59.767405768458715, 4.423502915866607], [-60.11100236676738, 4.574966538914083], [-59.980958624904886, 5.014061184098139], [-60.21368343773133, 5.244486395687602], [-60.73357418480372, 5.200277207861901], [-61.410302903881956, 5.959068101419618], [-61.13941504580795, 6.234296779806144], [-61.15933631045648, 6.696077378766319], [-60.54399919294099, 6.856584377464883], [-60.2956680975624, 7.043911444522919], [-60.637972785063766, 7.4149999048108555], [-60.5505879380582, 7.779602972846178], [-59.758284878159195, 8.367034816924047], [-59.10168412945866, 7.999201971870492], [-58.48296220562806, 7.347691351750697], [-58.45487606467742, 6.832787380394464], [-58.078103196837375, 6.809093736188643], [-57.542218593970645, 6.321268215353356], [-57.14743648947689, 5.973149929219161], [-57.307245856339506, 5.073566595882227], [-57.91428890647214, 4.812626451024414], [-57.8602095200787, 4.57680105226045], [-58.04469438336068, 4.0608635522583825], [-57.60156897645787, 3.3346546492606848], [-57.28143347840971, 3.3334919295341194], [-57.15009782573991, 2.7689269067454063], [-56.539385748914555, 1.8995226098669207]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-54.524754197799716, 2.3118488631237852], [-55.09758744975514, 2.5237480737366127], [-55.569755011606, 2.4215062524471307], [-55.973322109589375, 2.510363877773017], [-56.0733418442903, 2.2207949894254995], [-55.905600145070885, 2.0219957543986595], [-55.995698004771754, 1.8176671411166012], [-56.539385748914555, 1.8995226098669207], [-57.15009782573991, 2.7689269067454063], [-57.28143347840971, 3.3334919295341194], [-57.60156897645787, 3.3346546492606848], [-58.04469438336068, 4.0608635522583825], [-57.8602095200787, 4.57680105226045], [-57.91428890647214, 4.812626451024414], [-57.307245856339506, 5.073566595882227], [-57.14743648947689, 5.973149929219161], [-55.9493184067898, 5.772877915872002], [-55.841779751190415, 5.95312531170606], [-55.033250291551774, 6.025291449401664], [-53.9580446030709, 5.756548163267765], [-54.47863298197923, 4.896755682795586], [-54.399542202356514, 4.212611395683467], [-54.00693050801901, 3.6200377465925584], [-54.181726040246275, 3.1897797713304215], [-54.2697051662232, 2.7323916691150463], [-54.524754197799716, 2.3118488631237852]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-75.37322323271385, -0.1520317521204504], [-75.23372270374195, -0.9114169246495294], [-75.54499569365204, -1.5616097957458803], [-76.63539425322672, -2.6086776668438176], [-77.83790483265861, -3.003020521663103], [-78.45068396677564, -3.873096612161376], [-78.63989722361234, -4.547784112164074], [-79.20528906931773, -4.959128513207389], [-79.62497921417618, -4.454198093283495], [-80.02890804718561, -4.3460909969288934], [-80.44224199087216, -4.425724379090674], [-80.46929460317695, -4.0592867977089995], [-80.18401485870967, -3.8211617977080437], [-80.30256059438722, -3.4048564591647126], [-79.77029334178093, -2.65751189535964], [-79.98655921092242, -2.220794366061014], [-80.36878394236925, -2.6851587866357884], [-80.96776546906436, -2.246942640800704], [-80.76480628123804, -1.9650477026485331], [-80.93365902375172, -1.057454522306358], [-80.58337032746127, -0.9066626928786832], [-80.39932471385376, -0.28370330160014134], [-80.02089820018037, 0.3603400740534682], [-80.09060970734211, 0.7684288598623965], [-79.5427620103998, 0.982937730305963], [-78.85525875518871, 1.380923773601822], [-77.85506140817952, 0.8099250349927729], [-77.66861284047044, 0.8258930525709616], [-77.4249843004304, 0.395686753741117], [-76.5763797675494, 0.256935533037435], [-76.29231441924097, 0.4160472680641192], [-75.8014658271166, 0.08480133707320192], [-75.37322323271385, -0.1520317521204504]]]}",
          "{\"type\": \"Polygon\", \"coordinates\": [[[-58.166392381408045, -20.176700941653678], [-57.8706739976178, -20.73268767668195], [-57.937155727761294, -22.090175876557172], [-56.8815095689029, -22.28215382252148], [-56.47331743022939, -22.086300144135283], [-55.79795813660691, -22.356929620047822], [-55.610682745981144, -22.655619398694846], [-55.517639329639636, -23.571997572526637], [-55.40074723979542, -23.956935316668805], [-55.02790178080955, -24.00127369557523], [-54.65283423523513, -23.83957813893396], [-54.29295956075452, -24.02101409271073], [-54.29347632507745, -24.570799655863965], [-54.42894609233059, -25.162184747012166], [-54.625290696823576, -25.739255466415514], [-54.78879492859505, -26.621785577096134], [-55.69584550639816, -27.387837009390864], [-56.486701626192996, -27.548499037386293], [-57.60975969097614, -27.395898532828387], [-58.61817359071975, -27.123718763947096], [-57.63366004091113, -25.60365650808164], [-57.77721716981794, -25.16233977630904], [-58.80712846539498, -24.77145924245331], [-60.02896603050403, -24.032796319273274], [-60.846564704009914, -23.880712579038292], [-62.685057135657885, -22.249029229422387], [-62.291179368729225, -21.051634616787393], [-62.2659612697708, -20.513734633061276], [-61.78632646345377, -19.633736667562964], [-60.04356462262649, -19.342746677327426], [-59.11504248720611, -19.3569060197754], [-58.183471442280506, -19.868399346600363], [-58.166392381408045, -20.176700941653678]]]}"
        ]
      },
      "map_join": [["country"], ["name"]],
      "color": "white"
    }
  ]
}"""
            return parsePlotSpec(spec)
        }
    }
}
